!===============================================================
!
! Copyright (C) 2005 Finite Difference Research Group
! This file is part of parsec, http://www.ices.utexas.edu/parsec/
!
! Calculates g-vectors (kgv) for lattice points inside a G-grid.
! G-vectors are ordered by stars depending on symmetry and pairs of
! (+G,-G) vectors are put together. Notice that
! the G-grid is defined as (-kmax,kmax), and the number of grid points
! along each direction is odd by construction. But the FFT box is
! defined as (m,m + N - 1), where N is *even*. In the parent
! routine, kmax is defined so that 2*kmax + 1 >= N.
!
! Adapted from plane-wave programs written by S. Froyen and
! J. L. Martins.
!
!---------------------------------------------------------------
subroutine g_space(pbc,symm,kmax,ipr,ierr)

  use constants
  use pbc_module
  use symmetry_module

  implicit none
  !
  ! Input/Output variables:
  !
  ! periodic boundary conditions data
  type (pbc_data), intent(inout) :: pbc
  ! symmetry operations
  type (symmetry), intent(in) :: symm
  ! half-size of reciprocal space
  integer, intent(inout) :: kmax(3)
  ! print flag
  integer, intent(in) :: ipr
  ! error flag, 260 < ierr < 266
  integer, intent(out) :: ierr
  !
  ! Work variables:
  !
  integer ns,ng
  integer i,j,k,l,istar,istop,ir,itest,iprot,iavail,ncount
  integer kk(3)
  integer, dimension (:), allocatable :: izstar
  ! number of g-vectors per star (transfered to pbc%mstar)
  integer, dimension(:), allocatable :: tmstar
  ! kinetic energy for each star (transfered to pbc%ek)
  real(dp), dimension(:), allocatable :: tek

  integer ktran(3,symm%ntrans)

  real(dp) :: gmod,rgm,gmr,dkk(3),dkt(3)

  real(dp) :: fi,diff,bdot(3,3)
  complex(dpc) :: phtmp
  complex(dpc) :: expt(symm%ntrans)
  real(dp), dimension (:), allocatable :: gm

  real(dp), parameter :: delta = 1.0d-8

  !---------------------------------------------------------------

  bdot = pbc%bdot

  ! g = k1*b1+k2*b2+k3*b3  

  ng=(2*kmax(1)+1)*(2*kmax(2)+1)*(2*kmax(3)+1)

  if(ng <= 1) then
     write(7,56) ng
     ierr = 261
     return
  endif

  if (.not. allocated (gm)) allocate (gm(ng))
  allocate(tek(ng))
  allocate(tmstar(ng))
  call pbc_set_ng (ng,pbc)

  ng = 0
  do i=-kmax(1),kmax(1)
     dkk(1) = real(i,dp)
     do j=-kmax(2),kmax(2)
        dkk(2) = real(j,dp)
        do k=-kmax(3),kmax(3)
           dkk(3) = real(k,dp)
           call matvec3('N',bdot,dkk,dkt)
           gmod = dot_product(dkk,dkt)
           ng = ng + 1
           gm(ng) = gmod
           pbc%kgv(1,ng) = i
           pbc%kgv(2,ng) = j
           pbc%kgv(3,ng) = k
        enddo
     enddo
  enddo

  ! Sort the g-vectors according to length and collect
  l=ng/2+1
  ir=ng
20 continue

  if(l > 1)then
     l=l-1
     rgm=gm(l)
     kk(:)=pbc%kgv(:,l)
  else
     rgm=gm(ir)
     kk(:)=pbc%kgv(:,ir)
     gm(ir)=gm(1)
     pbc%kgv(:,ir)=pbc%kgv(:,1)
     ir=ir-1
     if(ir == 1)then
        gm(1)=rgm
        pbc%kgv(:,1)=kk(:)
        goto 25
     
     endif
  endif
  i=l
  j=l+l

21 continue

  if(j <= ir)then
     if(j < ir)then
        if(gm(j) < gm(j+1))j=j+1
     endif
     if(rgm < gm(j))then
        gm(i)=gm(j)
        pbc%kgv(:,i)=pbc%kgv(:,j)
        i=j
        j=j+j
     else
        j=ir+1
     endif

     go to 21

  endif
  gm(i)=rgm
  pbc%kgv(:,i)=kk(:)

  go to 20

25 continue
  !
  ! Sort g vectors of same length by the kgv(1,i), then kgv(2,i),
  ! then the kgv(3,i) value. this makes the kgv sort machine
  ! independent.
  !
  istar = 2

  do i=2,ng
     if( i == ng) then
        diff = 1.0
     else
        diff = gm(i+1)-gm(istar)
     endif
     if(diff > delta) then
        istop = i
        do j=istar,istop-1
           do k=j+1,istop
              if (pbc%kgv(1,k) < pbc%kgv(1,j)) then
                 rgm = gm(j)
                 kk(:) = pbc%kgv(:,j)
                 gm(j) = gm(k)
                 pbc%kgv(:,j) = pbc%kgv(:,k)
                 gm(k) = rgm
                 pbc%kgv(:,k)=kk(:)
              elseif (pbc%kgv(1,k) == pbc%kgv(1,j)) then
                 if (pbc%kgv(2,k) < pbc%kgv(2,j)) then
                    rgm = gm(j)
                    kk(:) = pbc%kgv(:,j)
                    gm(j) = gm(k)
                    pbc%kgv(:,j) = pbc%kgv(:,k)
                    gm(k) = rgm
                    pbc%kgv(:,k)=kk(:)
                 elseif (pbc%kgv(2,k) == pbc%kgv(2,j)) then
                    if (pbc%kgv(3,k) < pbc%kgv(3,j)) then
                       rgm = gm(j)
                       kk(:) = pbc%kgv(:,j)
                       gm(j) = gm(k)
                       pbc%kgv(:,j) = pbc%kgv(:,k)
                       gm(k) = rgm
                       pbc%kgv(:,k)=kk(:)
                    endif
                 endif
              endif
           enddo
        enddo
        istar = istop + 1
     endif
  enddo
  !
  ! collects g-vectors into stars by symmetry
  ! first star is (0 0 0)
  !
  ns = 1
  tek(1) = zero
  pbc%phase(1) = zone
  pbc%conj(1) = one
  pbc%inds(1) = 1
  tmstar(1) = 1

  iprot = 1
  iavail = 2
  itest = 2

30 continue

  if(itest > ng) then
     diff = one
  else
     diff = abs(gm(itest)-gm(iprot))
  endif
  if(diff > delta) then
     ! new star
     iprot = iavail
     ns = ns + 1
     tek(ns) = gm(iprot) / two
     phtmp = zzero
     ncount = 0
     do i=1,symm%ntrans
        do j = 1, 3
           ktran(j,i) = dot_product(symm%gmtrx(j,:,i),pbc%kgv(:,iprot))
        enddo
        fi = dot_product(symm%tnp(:,i),ktran(:,i)*twopi)
        expt(i) = cmplx(cos(fi),sin(fi),dp)
        if ((ktran(1,i)-pbc%kgv(1,iprot) == 0) .and. &
             (ktran(2,i)-pbc%kgv(2,iprot) == 0) .and. &
             (ktran(3,i)-pbc%kgv(3,iprot) == 0)) then
           phtmp = phtmp + expt(i)
           ncount = ncount + 1
        endif
     enddo
     pbc%phase(iprot) = phtmp/real(ncount,dp)
     pbc%conj(iprot) = one
     tmstar(ns) = 1
     pbc%inds(iprot) = ns

     ! searches for the inverse of star member
     do i=iprot+1,ng
        if ((pbc%kgv(1,i)+pbc%kgv(1,iprot) == 0) .and. &
             (pbc%kgv(2,i)+pbc%kgv(2,iprot) == 0) .and. &
             (pbc%kgv(3,i)+pbc%kgv(3,iprot) == 0)) then
           if(i /= iprot+1) then
              kk(:) = pbc%kgv(:,iprot+1)
              pbc%kgv(:,iprot+1) = pbc%kgv(:,i)
              pbc%kgv(:,i) = kk(:)
              gmr = gm(iprot+1)
              gm(iprot+1) = gm(i)
              gm(i) = gmr
           endif
           !
           ! verifies if the inverse is related by symmetry
           !
           ncount = 0
           phtmp = zzero
           do j=1,symm%ntrans
              if ((ktran(1,j)-pbc%kgv(1,iprot+1) == 0) .and. &
                   (ktran(2,j)-pbc%kgv(2,iprot+1) == 0) .and. &
                   (ktran(3,j)-pbc%kgv(3,iprot+1) == 0)) then
                 phtmp = phtmp + expt(j)
                 ncount = ncount + 1
              endif
           enddo

           if(ncount /= 0) then
              pbc%phase(iprot+1) = phtmp/real(ncount,dp)
              pbc%conj(iprot+1) = one
           else
              pbc%phase(iprot+1) = pbc%phase(iprot)
              pbc%conj(iprot+1) = -one
           endif

           tmstar(ns) = tmstar(ns) + 1
           pbc%inds(iprot+1) = ns

           goto 31

        endif
     enddo

31   continue

     iavail = iprot + 2
     itest = iprot + 2

  else
     !
     ! checks if it is related by symmetry
     !
     ncount = 0
     phtmp = zzero
     do i=1,symm%ntrans
        if ((ktran(1,i)-pbc%kgv(1,itest) == 0) .and. &
             (ktran(2,i)-pbc%kgv(2,itest) == 0) .and. &
             (ktran(3,i)-pbc%kgv(3,itest) == 0)) then
           phtmp = phtmp + expt(i)
           ncount = ncount + 1
        endif
     enddo

     if(ncount /= 0) then
        !
        ! it is related by symmetry
        !
        if(itest /= iavail) then
           kk(:) = pbc%kgv(:,iavail)
           pbc%kgv(:,iavail) = pbc%kgv(:,itest)
           pbc%kgv(:,itest) = kk(:)
           gmr = gm(iavail)
           gm(iavail) = gm(itest)
           gm(itest) = gmr
        endif
        pbc%phase(iavail) = phtmp/real(ncount,dp)
        pbc%conj(iavail) = one
        tmstar(ns) = tmstar(ns) + 1
        pbc%inds(iavail) = ns
        !
        ! searches for the inverse of star member
        !
        do i=iavail+1,ng
           if ((pbc%kgv(1,i)+pbc%kgv(1,iavail) == 0) .and. &
                (pbc%kgv(2,i)+pbc%kgv(2,iavail) == 0) .and. &
                (pbc%kgv(3,i)+pbc%kgv(3,iavail) == 0)) then
              if(i /= iavail+1) then
                 kk(:) = pbc%kgv(:,iavail+1)
                 pbc%kgv(:,iavail+1) = pbc%kgv(:,i)
                 pbc%kgv(:,i) = kk(:)
                 gmr = gm(iavail+1)
                 gm(iavail+1) = gm(i)
                 gm(i) = gmr
              endif
              !
              ! verifies if the inverse is related by symmetry
              !
              ncount = 0
              phtmp = zzero
              do j=1,symm%ntrans
                 if ((ktran(1,j)-pbc%kgv(1,iavail+1) == 0) .and. &
                      (ktran(2,j)-pbc%kgv(2,iavail+1) == 0) .and. &
                      (ktran(3,j)-pbc%kgv(3,iavail+1) == 0)) then
                    phtmp = phtmp + expt(j)
                    ncount = ncount + 1
                 endif
              enddo

              if(ncount /= 0) then
                 pbc%phase(iavail+1) = phtmp/real(ncount,dp)
                 pbc%conj(iavail+1) = one
              else
                 pbc%phase(iavail+1) = pbc%phase(iavail)
                 pbc%conj(iavail+1) = -one
              endif
              tmstar(ns) = tmstar(ns) + 1
              pbc%inds(iavail+1) = ns

              goto 32

           endif
        enddo

32      continue

        if(itest == iavail) then
           itest = itest +2
        else
           itest = itest +1
        endif
        iavail = iavail +2

     else
        !
        ! increases test counter
        !
        itest = itest + 1
     endif

  endif

  if(iavail <= ng) goto 30
  !
  ! reset kmax 
  !
  kmax = maxval(pbc%kgv,dim=2)
  !
  ! checks stars with zero phase factors
  !
  if (.not. allocated (izstar)) allocate (izstar (1:ns))
  ncount = 1
  izstar(1) = 1
  do i=2,ns
     if(abs(pbc%phase(ncount+1)-one) > delta) then
        do j=ncount+1,ncount+tmstar(i)
           pbc%phase(j) = zzero
        enddo
        izstar(i) = 0
     else
        izstar(i) = 1
     endif
     ncount = ncount + tmstar(i)
  enddo
  !
  ! Save information of stars
  !
  call pbc_set_nstar (pbc,ns)
  pbc%mstar(1:pbc%nstar) = tmstar(1:pbc%nstar)
  call dcopy(pbc%nstar,tek,1,pbc%ek,1)

  write(7,50) ng,ns,(kmax(i),i=1,3)
  if (ipr >= 5) then
     !
     ! print g-vectors
     !
     istop = 0
     do i=1,pbc%nstar
        write(7,52) i,pbc%mstar(i),pbc%ek(i),izstar(i)
        istar = istop+1
        istop = istar+pbc%mstar(i)-1
        do j=istar,istop
           write(7,54) j,pbc%inds(j),(pbc%kgv(k,j),k=1,3), &
                real(pbc%phase(j)),aimag(pbc%phase(j)),pbc%conj(j)
        enddo
     enddo
  endif

  deallocate (gm)
  deallocate (izstar)
  deallocate(tmstar)
  deallocate(tek)

50 format(/,1x,i8,' g-vectors are set up in ',i7,' stars', &
        ' - kmax = ',3i4)
52 format(/,' star no',i4,'  with',i3,' members - kin energy =', &
        f12.8,4x,i3,//, &
        4x,'i',3x,'inds',5x,'kx',3x,'ky',3x,'kz',9x,'phase')
54 format(i5,2x,i5,2x,3i5,f10.5,3x,f10.5,5x,f5.2)
56 format('  *** stopped in gspace   no. of g-vec.= ',i6)

end subroutine g_space
!===============================================================
