!===============================================================
!
! Copyright (C) 2005 Finite Difference Research Group
! This file is part of parsec, http://www.ices.utexas.edu/parsec/
!
! This subroutine checks whether there are insulated atoms -
! i.e., atoms whose position prevents any bonding with any other
! atoms and may therefore lead to non-physical results.
!
! It must be placed INSIDE the atom movement loop, so that 
! the atom positions  will be rechecked after each molecular 
! dynamics or energy minimization step.
!
! WARNING!
! At present the only allowed grid setup involves one sphere 
! encompassing the entire cluster. Therefore the only check
! needed is that no atom is outside this boundary sphere. Should
! other grid constructions (e.g., an individual sphere around
! each atom) be implemented, the test(s) should be updated
! accordingly.
!
!---------------------------------------------------------------
subroutine isolat(clust,rmax,nper,ierr)

  use constants
  use cluster_module 
  implicit none
  !
  ! Input/Output variables:
  !
  ! the cluster
  type (cluster), intent(in) :: clust
  ! boundary sphere size 
  real(dp), intent(in) :: rmax
  ! Number of periodic directions, the first nper directions in the
  ! set (x,y,z) are assumed to be periodic.
  integer, intent(in) :: nper
  ! error flag, 290 < ierr < 301
  integer, intent(out) :: ierr
  !
  ! Work variables:
  !
  ! index which is false if all is well, true if any isolated atom found
  logical isolated_found 
  ! distance of coordinate from origin
  real(dp) :: dist 
  ! counter
  integer i
  ! temp r-vactor 
  real(dp) :: r_vec (3)
  !---------------------------------------------------------------
  !
  ! For each atom, find its distance from the origin, if this
  ! distance is larger than rmax, flag a problem for the specific
  ! atom and for the whole cluster.
  !
  isolated_found = .false.
  do i=1, clust%atom_num
     r_vec = (/clust%xatm(i),clust%yatm(i),clust%zatm(i)/)
     r_vec(1:nper) = zero
     dist = sqrt (dot_product (r_vec,r_vec))
     if (dist > rmax) then 
        if (.not. isolated_found) then
           isolated_found = .true.
           write(7,'(/,a,/)') 'ERROR: isolated atoms found:'
        endif
        write(7,10) i
     endif
  enddo

  if (isolated_found) then
     ! If problem found - report it and terminate.
     write(7,*) 'STOP in isolat.'
     ierr = 291
     return
  endif

10 format(1x,'Atom #',i4,2x,'is outside the boundary sphere!')

end subroutine isolat
!===============================================================
